/*  This program implements the dialog box of figure 7.3 in
    "Writing GNOME Applications by John R. Sheets.  Function-
    ality was added to determine which button was pushed, and
    what text the user entered.  This required looking at the
    definitions of structures in the GTK library, the GLIB
    library, and the GNOME library; I got these off the web 
    by downloading the tarballs of the API documentation, 
    which is in html format when uncompressed, from 
    www.gtk.org and www.gnome.org. 								
    
    I used automake, autoconf, etc..., to compile, so the
    PACKAGE and VERSION are defined there.					*/

#include <gnome.h>

void quit_cb(GtkWidget *widget)
{
   g_message("Quitting...\n");
   gtk_main_quit();
}


void call_dbutton_cb(GnomeDialog *dialog, gint arg1, gpointer nada)
{
   /* Here's the meat on how to determine the user's interaction 
      with the dialog...										*/
      	
   GtkWidget *child, *gtkentry;
   GList *Gltemp;
   char *text;
      
   /* arg1, listed in the "clicked" signal call back function
      prototype in the gnomeui library documentation, tells us
      which button was clicked.									*/

   g_message("You pushed button %d! \n", arg1);
   
   /* Gltemp becomes a pointer to the linked list holding the 
      GtkBoxChild members in the line of code below.  The
      GtkBox structure looks like this:
      struct GtkBox
      {  GList *children; * a pointer to a Glist of children.*
         gint16 spacing; 
         guint homogeneous;
      } 													*/
	/*  {         pointer to GtkBox          }{ convert to
	    {                                    }{ pointer to
	    {                                    }{ GList   }  */
   Gltemp = ((GTK_BOX(GNOME_DIALOG(dialog)->vbox))->children);
   
   /* get 4th item in list--the entry box */
   Gltemp = g_list_nth(Gltemp, (guint)3);

   /* Get pointer to entry widget from the pointer to 
      GtkBoxChild struct child will point to the entry 
      widget.  The GList structure is defined as:
      struct GList
      {
         gpointer data; * this is the void pointer we will
         			* cast to a GtkBoxChild pointer.
         GList *next;
         GList *prev;
      }
      The GtkBoxChild structure is defined as:
      struct GtkBoxChild
      {
         GtkWidget *widget; *the child widget of our GtkBox
         ...
      } 												*/

   child = ((GtkBoxChild *)(Gltemp->data))->widget;
   
   /* get the GtkEntry widget out of the child so we can
      get the text from it using the GTK library.  The
      GnomeEntry functions can't do this.		        	*/
   gtkentry = gnome_entry_gtk_entry(GNOME_ENTRY(child));
   /* retrieve the contents... */
   text = gtk_entry_get_text(GTK_ENTRY(gtkentry));
   /* Ta Duh!  We have pulled the text from our custom
      dialog...												*/
   g_message("The text is %s",text);
   	
}

void call_dialog_cb(GtkButton* button, gpointer widget)
{

/* *************************************************************
   Note:  I tried creating the dialog in main(), but when I tried
   	  to pass in the pointer as widget, which is cast as a
   	  void pointer (no way around this that I could find), the
   	  cast to GnomeDialog using GNOME_DIALOG(widget) failed.
   	  So, I was stuck creating the dialog here.
***************************************************************/   	  

   /* See Listing 7.3 on pg. 198--most of this function came
      from there.											 */

   GtkWidget *dialog, *label, *entry, *d_button;
   GList *Gltemp;

   dialog = gnome_dialog_new("Name Dialog", GNOME_STOCK_BUTTON_OK,
      GNOME_STOCK_BUTTON_CANCEL, NULL);
   label = gtk_label_new("Please enter your name");
 
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
      label, TRUE, TRUE, 0);
   entry = gnome_entry_new("name_history_id");
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
      entry, TRUE, TRUE, 0);
   
   /* I found that the gnome_dialg_run() function would not
      close the dialog, nor would it display the label or
      the entry box.  Also, since I wanted to hook into the
      "clicked" signal for the dialog, I chose to use 
      gtk_widget_show_all() (gtk_widget_show() didn't display
      the label or entry box either).						  */
  
   gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
   gnome_dialog_set_close(GNOME_DIALOG(dialog),TRUE);
   gtk_widget_show_all(GTK_WIDGET(dialog));
  
   /* Here is my first attempt at figuring out how to access
      the child widgets (see the GTK online API documentation 
      for the structure definition).  This showed me that there
      were only the four children I expected: two buttons, a 
      label and an entry box									*/

   Gltemp = (GTK_BOX(GNOME_DIALOG(dialog)->vbox))->children;
   g_message("Number of members in GList is %u",g_list_length(Gltemp));

   /* clicking any button in the dialog causes this "clicked" 
      signal event.  I connected it to the call_dbutton0_cb()
      function, where we do the real work of determining the
      users actions within the dialog.  Note that because we
      set the boolean to TRUE in gnome_dialog_set_close(), the
      dialog will close itself when a button is clicked; 
      however, it will still send this signal first.			*/

   gtk_signal_connect(GTK_OBJECT(dialog), "clicked",
      GTK_SIGNAL_FUNC(call_dbutton_cb), NULL);  
}

int main(int argc, char *argv[])
{
   /* Building on the code from listing 5.2, pg. 138 */

   GtkWidget *app, *button;
   
   gnome_init(PACKAGE, VERSION, argc, argv);
   
   g_message("Creating app...\n");
   app = gnome_app_new(PACKAGE, "My GNOME Application");
   
   button = gtk_button_new_with_label("Open Dialog");
   gnome_app_set_contents(GNOME_APP(app), button);
   
   g_message("Connecting signals...\n");
   gtk_signal_connect(GTK_OBJECT(app), "delete_event",
      GTK_SIGNAL_FUNC(quit_cb), NULL);

   /* Connect the button to a new function that will
      bring up a custom dialog box                   */
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
      GTK_SIGNAL_FUNC(call_dialog_cb), NULL);
   /* The My GNOME Application title was not showing up because the
      application window was not big enough.  The following call
      resized the application, with the first number below the width,
      the second number the height */
   gtk_window_set_default_size(GTK_WINDOW(app),(gint)250,(gint)50);
   gtk_widget_show_all(app);
   

      
   g_message("Entering main loop...\n");
   gtk_main();
   
   return 0;
}
